@@ -609,20 +609,17 @@ int check_chan_user_modes(const char *source, User *user, const char *chan,
     /* Mode additions.  Only check for join or server mode change, unless
      * ENFORCE is set */
     if (is_servermode || (ci->flags & CI_ENFORCE)) {
-	int set = 0;
 #ifdef HAVE_CHANPROT
 	if (is_founder(user, ci)) {
 	    ci->last_used = time(NULL);
 	    if (!(modes & CUMODE_q))
 		send_cmode(MODE_SENDER(s_ChanServ), chan, "+q", user->nick);
 	    modes |= CUMODE_q;
-	    set++;
 	} else if (check_access(user, ci, CA_AUTOPROTECT)) {
 	    ci->last_used = time(NULL);
 	    if (!(modes & CUMODE_a))
 		send_cmode(MODE_SENDER(s_ChanServ), chan, "+a", user->nick);
 	    modes |= CUMODE_a;
-	    set++;
 	}
 #endif
 	if (check_access(user, ci, CA_AUTOOP)) {
@@ -630,22 +627,17 @@ int check_chan_user_modes(const char *source, User *user, const char *chan,
 	    if (!(modes & CUMODE_o))
 		send_cmode(MODE_SENDER(s_ChanServ), chan, "+o", user->nick);
 	    modes |= CUMODE_o;
-	    set++;
 #ifdef HAVE_HALFOP
 	} else if (check_access(user, ci, CA_AUTOHALFOP)) {
 	    if (!(modes & CUMODE_h))
 		send_cmode(MODE_SENDER(s_ChanServ), chan, "+h", user->nick);
 	    modes |= CUMODE_h;
-	    set++;
 #endif
 	} else if (check_access(user, ci, CA_AUTOVOICE)) {
 	    if (!(modes & CUMODE_v))
 		send_cmode(MODE_SENDER(s_ChanServ), chan, "+v", user->nick);
 	    modes |= CUMODE_v;
-	    set++;
 	}
-	if (set)
-	    return modes;  /* don't bother checking for mode subtractions */
     }
 
     /* Don't subtract modes from opers or Services admins */
@@ -664,7 +656,7 @@ int check_chan_user_modes(const char *source, User *user, const char *chan,
 	    modes &= ~CUMODE_h;
 	}
 #endif
-#ifdef HAVE_HALFOP
+#ifdef HAVE_CHANPROT
 	if (modes & CUMODE_a) {
 	    send_cmode(MODE_SENDER(s_ChanServ), chan, "-a", user->nick);
 	    modes &= ~CUMODE_a;
