@@ -1,6 +1,6 @@
 /* Prototypes and external variable declarations.
  *
- * IRC Services is copyright (c) 1996-2001 Andrew Church.
+ * IRC Services is copyright (c) 1996-2002 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts copyright (c) 1999-2000 Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -32,6 +32,9 @@ E void load_akill(void);
 E void save_akill(void);
 E int check_akill(const char *nick, const char *username, const char *host);
 E void expire_akills(void);
+#ifdef IRC_UNREAL
+E void m_tkl(char *source, int ac, char **av);
+#endif
 E void do_akill(User *u);
 E void add_akill(char *mask, const char *reason, const char *who,
 		 const time_t expiry);
