@@ -1,6 +1,6 @@
 /* Multi-language support.
  *
- * IRC Services is copyright (c) 1996-2001 Andrew Church.
+ * IRC Services is copyright (c) 1996-2002 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts copyright (c) 1999-2000 Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -24,7 +24,8 @@ int langlist[NUM_LANGS];
 /* Order in which languages should be displayed: (alphabetical) */
 static int langorder[] = {
     LANG_EN_US,		/* English (US) */
-    LANG_FR,		/* French */
+    LANG_NL,		/* Dutch */
+/*    LANG_FR,*/	/* French */
     LANG_DE,		/* German */
     LANG_IT,		/* Italian */
 /*    LANG_JA_JIS,*/	/* Japanese (JIS encoding) */
@@ -35,6 +36,21 @@ static int langorder[] = {
     LANG_TR,		/* Turkish */
 };
 
+/* Filenames for language files: */
+static const char *filenames[] = {
+    [LANG_EN_US]	"en_us",
+    [LANG_NL]		"nl",
+    [LANG_FR]		"fr",
+    [LANG_DE]		"de",
+    [LANG_IT]		"it",
+/*    [LANG_JA_JIS]	"ja_jis",*/
+    [LANG_JA_EUC]	"ja_euc",
+    [LANG_JA_SJIS]	"ja_sjis",
+    [LANG_PT]		"pt",
+    [LANG_ES]		"es",
+    [LANG_TR]		"tr",
+};
+
 /*************************************************************************/
 
 /* Load a language file. */
@@ -127,15 +143,8 @@ void lang_init()
 {
     int i, j, n = 0;
 
-    load_lang(LANG_EN_US, "en_us");
-    load_lang(LANG_DE, "de");
-    load_lang(LANG_IT, "it");
-/*    load_lang(LANG_JA_JIS, "ja_jis"); */
-    load_lang(LANG_JA_EUC, "ja_euc");
-    load_lang(LANG_JA_SJIS, "ja_sjis");
-    load_lang(LANG_ES, "es");
-    load_lang(LANG_PT, "pt");
-    load_lang(LANG_TR, "tr");
+    for (i = 0; i < lenof(langorder); i++)
+	load_lang(langorder[i], filenames[langorder[i]]);
 
     if (!langtexts[DEF_LANGUAGE])
 	fatal("Unable to load default language");
