@@ -1,6 +1,6 @@
 /* NickServ functions.
  *
- * IRC Services is copyright (c) 1996-2001 Andrew Church.
+ * IRC Services is copyright (c) 1996-2002 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts copyright (c) 1999-2000 Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -60,6 +60,7 @@ static const char hashtable[256] = {
 
 #define TO_COLLIDE   0			/* Collide the user with this nick */
 #define TO_RELEASE   1			/* Release a collided nick */
+#define TO_SEND_433  2			/* Send a 433 numeric */
 
 /*************************************************************************/
 
@@ -397,6 +398,9 @@ int validate_user(User *u)
 	else
 #endif
 	    notice_lang(s_NickServ, u, DISCONNECT_IN_1_MINUTE);
+#ifndef DONT_SEND_433
+	add_ns_timeout(ni, TO_SEND_433, 40);
+#endif
 	add_ns_timeout(ni, TO_COLLIDE, 60);
 	return 0;
     }
@@ -409,10 +413,10 @@ int validate_user(User *u)
 	 *
 	 * Under ircd.dal 4.4.15+ and other servers supporting a Services
 	 * stamp, Services guarantees that the first condition cannot occur
-	 * unless the stamp counter rolls over (2^32-1 client connections).
+	 * unless the stamp counter rolls over (2^31-1 client connections).
 	 * This is practically infeasible.  As an example, on a network of
 	 * 30 servers, an attack introducing 10 new clients every second on
-	 * every server would need to be sustained for 166 days (5.5 months)
+	 * every server would need to be sustained for 83 days (2.8 months)
 	 * to cause the stamp to roll over.
 	 *
 	 * Under other servers, an attack is theoretically possible, but
@@ -485,6 +489,9 @@ int validate_user(User *u)
 #endif
 	    	notice_lang(s_NickServ, u, DISCONNECT_IN_20_SECONDS);
 	    add_ns_timeout(ni, TO_COLLIDE, 20);
+#ifndef DONT_SEND_433
+	    add_ns_timeout(ni, TO_SEND_433, 10);
+#endif
 	} else {
 #ifdef HAVE_NICKCHANGE
 	    if (NSForceNickChange)
@@ -493,6 +500,9 @@ int validate_user(User *u)
 #endif
 	    	notice_lang(s_NickServ, u, DISCONNECT_IN_1_MINUTE);
 	    add_ns_timeout(ni, TO_COLLIDE, 60);
+#ifndef DONT_SEND_433
+	    add_ns_timeout(ni, TO_SEND_433, 40);
+#endif
 	}
     }
 
@@ -1009,7 +1019,30 @@ static void timeout_release(Timeout *t)
 
 /*************************************************************************/
 
-/* Add a collide/release timeout. */
+/* Send a 433 (nick in use) numeric to the given user. */
+
+static void timeout_send_433(Timeout *t)
+{
+    NickInfo *ni = t->data;
+    User *u;
+
+    rem_ns_timeout(ni, TO_SEND_433, 0);
+    /* If they identified or don't exist anymore, don't send the 433. */
+    if ((ni->status & NS_IDENTIFIED)
+		|| !(u = finduser(ni->nick))
+		|| u->my_signon > t->settime)
+	return;
+    if (ni->status & NS_VERBOTEN)
+	send_cmd(ServerName, "433 %s %s :Nickname may not be used",
+		 ni->nick, ni->nick);
+    else
+	send_cmd(ServerName, "433 %s %s :Nickname is registered to someone"
+		 " else", ni->nick, ni->nick);
+}
+
+/*************************************************************************/
+
+/* Add a collide/release/433 timeout. */
 
 void add_ns_timeout(NickInfo *ni, int type, time_t delay)
 {
@@ -1021,6 +1054,8 @@ void add_ns_timeout(NickInfo *ni, int type, time_t delay)
 	timeout_routine = timeout_collide;
     else if (type == TO_RELEASE)
 	timeout_routine = timeout_release;
+    else if (type == TO_SEND_433)
+	timeout_routine = timeout_send_433;
     else {
 	log("%s: unknown timeout type %d!  ni=%p (%s), delay=%ld",
 	    s_NickServ, type, ni, ni->nick, delay);
@@ -1257,7 +1292,7 @@ static void do_identify(User *u)
     } else if (!(ni = u->real_ni)) {
 	notice(s_NickServ, u->nick, "Your nick isn't registered.");
 
-    } else if (ni->suspendinfo) {
+    } else if (getlink(ni)->suspendinfo) {
 	notice_lang(s_NickServ, u, NICK_X_SUSPENDED, u->nick);
 
     } else if (!(res = check_password(pass, ni->pass))) {
@@ -1545,16 +1580,16 @@ static void do_set_language(User *u, NickInfo *ni, char *param)
 
 static void do_set_url(User *u, NickInfo *ni, char *param)
 {
+    if (param && !valid_url(param)) {
+	notice_lang(s_NickServ, u, BAD_URL);
+	return;
+    }
+
     if (ni->url)
 	free(ni->url);
     if (param) {
-	if (!valid_url(param)) {
-	    ni->url = NULL;
-	    notice_lang(s_NickServ, u, BAD_URL);
-	} else {
-	    ni->url = sstrdup(param);
-	    notice_lang(s_NickServ, u, NICK_SET_URL_CHANGED, ni->nick, param);
-	}
+	ni->url = sstrdup(param);
+	notice_lang(s_NickServ, u, NICK_SET_URL_CHANGED, ni->nick, param);
     } else {
 	ni->url = NULL;
 	notice_lang(s_NickServ, u, NICK_UNSET_URL, ni->nick);
@@ -1565,18 +1600,18 @@ static void do_set_url(User *u, NickInfo *ni, char *param)
 
 static void do_set_email(User *u, NickInfo *ni, char *param)
 {
+    if (param && !valid_email(param)) {
+	notice_lang(s_NickServ, u, BAD_EMAIL);
+	return;
+    }
+
     if (ni->email)
 	free(ni->email);
     if (param) {
-	if (!valid_email(param)) {
-	    ni->email = NULL;
-	    notice_lang(s_NickServ, u, BAD_EMAIL);
-	} else {
-	    ni->email = sstrdup(param);
-	    log("%s: %s E-mail address changed to %s by %s!%s@%s",
-		s_NickServ, ni->nick, param, u->nick, u->username, u->host);
-	    notice_lang(s_NickServ, u, NICK_SET_EMAIL_CHANGED, ni->nick,param);
-	}
+	ni->email = sstrdup(param);
+	log("%s: %s E-mail address changed to %s by %s!%s@%s",
+	    s_NickServ, ni->nick, param, u->nick, u->username, u->host);
+	notice_lang(s_NickServ, u, NICK_SET_EMAIL_CHANGED, ni->nick,param);
     } else {
 	ni->email = NULL;
 	log("%s: %s E-mail address cleared by %s!%s@%s",
@@ -1858,7 +1893,10 @@ static void do_link(User *u)
 	}
 
 	/* Check for exceeding the channel registration limit. */
-	if (check_channel_limit(top) >= 0) {
+	top->channelcount += ni->channelcount;
+	res = check_channel_limit(top);
+	top->channelcount -= ni->channelcount;
+	if (res >= 0) {
 	    notice_lang(s_NickServ, u, NICK_LINK_TOO_MANY_CHANNELS, nick,
 			top->channelmax ? top->channelmax : MAX_CHANNELCOUNT);
 	    return;
@@ -2463,12 +2501,14 @@ static void do_forbid(User *u)
 {
     NickInfo *ni;
     char *nick = strtok(NULL, " ");
+    User *u2;
 
     /* Assumes that permission checking has already been done. */
     if (!nick) {
 	syntax_error(s_NickServ, u, "FORBID", NICK_FORBID_SYNTAX);
 	return;
     }
+    u2 = finduser(nick);
     if ((ni = findnick(nick)) != NULL) {
 	if (NSSecureAdmins && nick_is_services_admin(ni) &&
                                                         !is_services_root(u)) {
@@ -2476,20 +2516,24 @@ static void do_forbid(User *u)
 	    return;
 	}
 	delnick(ni);
+	if (u2)
+	    u2->ni = u2->real_ni = NULL;
     }
 
     if (readonly)
 	notice_lang(s_NickServ, u, READ_ONLY_MODE);
     ni = makenick(nick);
     if (ni) {
-	User *u2;
 	ni->status |= NS_VERBOTEN;
+	ni->language = DEF_LANGUAGE;
+	ni->time_registered = time(NULL);
 	log("%s: %s set FORBID for nick %s", s_NickServ, u->nick, nick);
 	notice_lang(s_NickServ, u, NICK_FORBID_SUCCEEDED, nick);
 	/* If someone is using the nick, make them stop */
-	u2 = finduser(nick);
-	if (u2)
+	if (u2) {
+	    u2->ni = u2->real_ni = ni;
 	    validate_user(u2);
+	}
     } else {
 	log("%s: Valid FORBID for %s by %s failed", s_NickServ,
 		nick, u->nick);
