@@ -1,6 +1,6 @@
 /* Main header for Services.
  *
- * IRC Services is copyright (c) 1996-2001 Andrew Church.
+ * IRC Services is copyright (c) 1996-2002 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts copyright (c) 1999-2000 Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -139,9 +139,10 @@ typedef struct memoinfo_ MemoInfo;
 #define LANG_FR		6	/* French */
 #define LANG_TR		7	/* Turkish */
 #define LANG_IT		8	/* Italian */
-#define LANG_DE		9
+#define LANG_DE		9	/* German */
+#define LANG_NL		10	/* Dutch */
 
-#define NUM_LANGS	10	/* Number of languages */
+#define NUM_LANGS	11	/* Number of languages */
 
 /* Sanity-check on default language value */
 #if DEF_LANGUAGE < 0 || DEF_LANGUAGE >= NUM_LANGS
